    AREA    Chaos89, CODE, READONLY

    ENTRY
    b   Start

    DCD 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    DCD 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
    DCD 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0

Start
    mov     r2, #0x3000000          ; Set Pointer to ram
    add     r2, r2, #0x60           ; Offset 0x60 is the current VRAM Page
    mov     r4, #0                  ; Set to zero
    str     r4, [r2]

    mov     r2, #0x3000000          ; Setup a pointer to Ram
    adrl    r1, Positions
    mov     r3, #12

    mov     r4, #0                  ; Set Sine offset to zero
    str     r4, [r2], #4

PositionLoop
    ldr     r4, [r1], #4
    str     r4, [r2], #4

    subs    r3, r3, #1
    bne     PositionLoop

    ldr     r3, PaletteDMA          ; Transfer Palette to Palette Ram
    mov     r0, #0x5000000          ; Using DMA #3 (0x0100 words)
    adrl    r1, Palette
    mov     r2, #0x4000000
    add     r2, r2, #0xd4

    str     r1, [r2], #0x04         ; Store the X positions in RAM
    str     r0, [r2], #0x04
    str     r3, [r2]

    ldr     r3, PictureDMA          ; Transfer the Image to VRAM 
    mov     r0, #0x6000000          ; Using DMA #3 (0x4800 words)
    adrl    r1, Image               ; Transfer from Image to 0x6000000
    mov     r2, #0x4000000          ; Transfer 0x4800 words
    add     r2, r2, #0xd4

    str     r1, [r2], #0x04         ; Transfer DMA parameters
    str     r0, [r2], #0x04         ; And start DMA
    str     r3, [r2]

    bl      DrawChaos               ; Draw the 'Chaos89.com text'

    mov     r2, #0x4000000          ; Turn on the screen and set mode 4
    mov     r1, #0x04               ; Set initial VRAM Page 1
    add     r1, r1, #0x400
    strh    r1, [r2]

Loop                                ; Endless loop
    mov     r2, #0x3000000          ; Update VRAM Page
    add     r2, r2, #0x60

    ldr     r12, [r2]
    add     r12, r12, #1

    cmp     r12, #2
    movge   r12, #0

    str     r12, [r2]

    ldr     r3, PictureDMA          ; Transfer the Image to VRAM 
    mov     r0, #0x6000000          ; Using DMA #3 (0x4800 words)

    cmp     r12, #1
    addge   r0, r0, #0xA000

    adrl    r1, Image
    mov     r2, #0x4000000
    add     r2, r2, #0xd4

VBlank
    mov     r6, #0x4000000          ; Wait for start of VBlank
    add     r6, r6, #0x04           
    ldrh    r5, [r6]

    and     r5, r5, #1

    cmp     r5, #1
    bne     VBlank

    str     r1, [r2], #0x04         ; Transfer DMA Parameters
    str     r0, [r2], #0x04         ; And start DMA once VBlank occurs
    str     r3, [r2]
    
    mov     r2, #0x4000000          ; Turn on the screen and set mode 4

    cmp     r12, #1                  

    movlt   r1, #0x04               ; Set correct VRAM Page
    movge   r1, #0x14

    add     r1, r1, #0x400
    strh    r1, [r2]

    bl      DrawChaos               ; Draw 'Chaos89.com' text

    b       Loop

DrawChaos           
    mov     r1, #12                 ; Draw 12 Letters
    adrl    r2, Chaos               ; Chaos Imagery
    mov     r3, #0x3000000          ; Ram Pointer

    ldr     r10, [r3]               ; Retrieve sine offset
    add     r10, r10, #1

    cmp     r10, #32                ; Reset to zero if >= 32
    subge   r10, r10, #32
    
    str     r10, [r3], #4

DrawLetters
    ldr     r5, [r3], #4            ; Load the X position of the letter
    adrl    r6, Sine                ; Load the Y position of the letter based
    add     r11, r10, r6            ; on the Sine position.
    ldrb    r8, [r11], #4            

    mov     r6, #240                ; Multiply Y position by 240 to get
    mul     r7, r8, r6              ; offset in VRAM Buffer
    add     r4, r7, r5              ; Calculate position in the Mask 
    adrl    r6, Mask                
    add     r4, r4, r6
    add     r8, r7, r5              ; Calculate position in graphics memory

    add     r8, r8, #0x6000000      ; Set pointer to either 0x6000000
                                    
    cmp     r12, #1                 ; or 0x600A000 based on VRAM Page
    addge   r8, r8, #0xA000

    mov     r5, #20                 ; Draw 20 lines 

OuterLoop
    mov     r6, #15                 ; Draw 15 pixels accross

InnerLoop
    ldrb    r7, [r2], #1            ; Load the graphics pixel
    ldrb    r9, [r4], #1            ; Load the mask value

    cmp     r9, #0                  ; Check the mask value
    bne     SkipPixel               ; Skip pixel if masked

    cmp     r7, #0                  ; Check if transparent pixel
    strneb  r7, [r8]                ; Store if not transparent

SkipPixel
    add     r8, r8, #1              ; Update graphics memory pointer

    subs    r6, r6, #1              ; Next pixel
    bne     InnerLoop

    add     r8, r8, #240-15         ; Skip to start of next line
    add     r4, r4, #240-15         ; Skip to start of next line in mask

    subs    r5, r5, #1              ; Next row
    bne     OuterLoop

    add     r10, r10, #4            ; Update Sine position, checking
    cmp     r10, #32                ; if value >= 32
    subge   r10, r10, #32

    subs    r1, r1, #1              ; Next letter
    bne     DrawLetters

    mov     pc, lr                  ; Return from subroutine

Palette
    INCBIN chaos.pal                ; BG Palette

PaletteDMA
    DCD     0x84000100              ; Size to DMA for palette

PictureDMA
    DCD     0x84004800              ; Size to DMA for picture

Image
    INCBIN chaos.raw                ; BG Image

Chaos                               ; 'Chaos89.com' letters
    INCBIN  c.raw                   
    INCBIN  h.raw
    INCBIN  a.raw
    INCBIN  o.raw
    INCBIN  s.raw
    INCBIN  8.raw
    INCBIN  9.raw
    INCBIN  dot.raw
    INCBIN  c.raw
    INCBIN  o.raw
    INCBIN  m.raw
    INCBIN  !.raw

Mask                            
    INCBIN  mask.raw                ; Mask image

Positions                           ; X Position of letters
    DCD     18                      
    DCD     35
    DCD     52
    DCD     69
    DCD     86
    DCD     103
    DCD     120
    DCD     137
    DCD     154
    DCD     171
    DCD     188
    DCD     205

Sine                                ; Sine table, Y Positions of the letters
    DCB  64,  66,  68,  70,  71,  72,  73,  74
    DCB  74,  74,  73,  72,  71,  70,  68,  66
    DCB  64,  62,  60,  58,  57,  56,  55,  54
    DCB  54,  54,  55,  56,  57,  58,  60,  62

    END
